<?php class Packs_Magento_Model_Export_Getlabels extends Mage_Core_Model_Abstract
{
    protected $apiUser;
    protected $apiUserPassword;
    protected $apiUrl;
    protected $apiAuthUrl;
    protected $oldTime;
    protected $orderIds;
    protected $postData;
    protected $shipments = array();
    protected $shipmentitems = array();
    protected $response;
    protected $responseMessages = array();
    protected $aToken;
    protected $tToken;
    protected $continue;
    protected $_labelsData;
    protected $_missingLabelOrderIds = array();

    public function __construct($shipmentids)
    {
        $this->shipments = $shipmentids;
        $this->preProcessingPdfCreation();
    }

    public function preProcessingPdfCreation(){

        $this->apiUser = Mage::getStoreConfig('packs_magento_general/general/api_user');
        $this->apiUserPassword = Mage::getStoreConfig('packs_magento_general/general/api_user_password');
        $this->apiUrl =  Mage::getStoreConfig('packs_magento_general/general/api_getlabels_url');
        $this->apiAuthUrl = Mage::getStoreConfig('packs_magento_general/general/api_auth_url');
        $this->continue = true;
        set_time_limit(720000);                 // script timeout: 15 mins.
        ini_set('display_errors', 1);
        error_reporting(E_ALL);
        ini_set('max_execution_time', 720000);
        ini_set("default_charset", "utf-8");
        ini_set("memory_limit", "-1");
        ini_set("display_errors", "on");
        ob_start();


        $this->processingPdfCreation();
    }

    public function processingPdfCreation()
    {


        Mage::log('Get Labels STARTED', NULL, 'getlabels.log');
        Mage::app()->setCurrentStore(Mage_Core_Model_App::ADMIN_STORE_ID);

        $tokenData = Mage::helper('packs_magento')->Authorize();
        $this->tToken = $tokenData['type'];
        $this->aToken = $tokenData['token'];
        Mage::log('time 0: Authorize : ' . $this->getSeconds() . ' sec', NULL, 'getlabels.log');

        $this->getPacksShipmentItems($this->shipments);

        $this->getLabelsFromApi($this->shipmentitems);
    }

    private function getPacksShipmentItems($shipments){
        $shipmentitemsCollection = Mage::getModel('packs_magento/shipmentitems')
            ->getCollection()
            ->addFieldToFilter('packs_shipment_id', array('in' => $shipments));
        $this->shipmentitems = $shipmentitemsCollection->getData();
    }


    public function getLabelsFromApi($packsShipments){

        $missingLabels = array();
            $shipmentId = $shipment['packs_shipment_id'];
            foreach ($packsShipments as $packsShipment){
                array_push($missingLabels, array('shipmentId' => $packsShipment['packs_shipment_id'], 'shipmentItemId' => $packsShipment['packs_shipment_item_id']));
            }

            try {
                // xml post structure
                $xml_post_array = array(
                    'mergeLabels' => true,
                    'shipmentItems' =>
                    $missingLabels
                );

                $xml_post_string = json_encode($xml_post_array);

                // PHP cURL  for https connection with auth
                $ch = curl_init();

                curl_setopt_array($ch, array(
                    CURLOPT_URL => "".$this->apiUrl."",
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_ENCODING => "",
                    CURLOPT_MAXREDIRS => 10,
                    CURLOPT_TIMEOUT => 30,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => "POST",
                    CURLOPT_POSTFIELDS => "".$xml_post_string."",
                    CURLOPT_HTTPHEADER => array(
                        "Accept: application/json",
                        "Authorization: ".$this->tToken." ".$this->aToken,
                        "Content-Type: application/json",
                        "cache-control: no-cache",
                    ),
                    CURLOPT_SSL_VERIFYPEER => false,

                ));

                $response = curl_exec($ch);

                if(empty($response)){
                    $response = curl_error($ch);
                    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                    curl_close($ch);
                    unset($ch);
                    Mage::getSingleton('core/session')->addError('Api response is empty');

                }else{

                    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                    if($httpCode == 404) {
                        $message = "HTTP Error 404. The requested resource is not found. URL";
                        $this->responseMessages['errors'][$order['increment_id']] = $message;

                    }
                    if($httpCode == 400) {
                        $message = $response;
                        $this->responseMessages['errors'][$order['increment_id']] = $message;

                    }
                    if($httpCode == 503) {
                        $message = "HTTP Error 503. The api server is not available";
                        $this->responseMessages['errors'][$order['increment_id']] = $message;

                    }
                    if (curl_errno($ch)) {
                        $err = curl_error($ch);
                        curl_close($ch);

                        $message = $err;
                        $this->responseMessages['errors'][$order['increment_id']] = $message;
                        exit();
                    }

                    $jsonDecoded = json_decode($response,true);
                    curl_close($ch);
                    unset($ch);

                    $this->_response = $jsonDecoded;
                }

            }catch (Exception $e) {
                Mage::log('time 4.1: ' . time(). ' : '.$e->getMessage(),NULL,'shipmentexport.log');
                Mage::logException($e);

                // send email
                if($this->sendErrorMail == true){
                    $this->helper->sendErrorMail($e->getMessage(),$this->sendErrorMailTo);
                }
            }
    }


    public function getSeconds(){
        if(!isset($this->oldTime)){
            $this->oldTime = time();
        }
        $newTime = time();
        $seconds = $newTime - $this->oldTime;
        $this->oldTime = $newTime;
        return $seconds;
    }

    public function setOrderIds()
    {
        $orderIds = Mage::getSingleton('packs_magento/export_data')->getData('orderIds');
        $this->orderIds = $orderIds;
    }

    public function setPostData()
    {
        $postData = Mage::getSingleton('packs_magento/export_data')->getData('postData');
        $this->postData = $postData;
    }

}