<?php class Packs_Magento_Model_Export_Shipment extends Mage_Core_Model_Abstract
{
    protected $apiUser;
    protected $apiUserPassword;
    protected $apiUrl;
    protected $apiAuthUrl;
    protected $oldTime;
    protected $orderIds;
    protected $postData;
    protected $orders = array();
    protected $responses = array();
    protected $responseMessages = array();
    protected $aToken;
    protected $tToken;
    protected $continue;

    public function startExport()
    {
        $this->preProcessingExport();
    }
    public function startMassExport()
    {
        $this->preProcessingExport();
    }
    public function preProcessingExport(){

        $this->apiUser = Mage::getStoreConfig('packs_magento_general/general/api_user');
        $this->apiUserPassword = Mage::getStoreConfig('packs_magento_general/general/api_user_password');
        $this->apiUrl = Mage::getStoreConfig('packs_magento_general/general/api_url');
        $this->apiAuthUrl = Mage::getStoreConfig('packs_magento_general/general/api_auth_url');
        $this->continue = true;
        set_time_limit(720000);                 // script timeout: 15 mins.
        ini_set('display_errors', 1);
        error_reporting(E_ALL);
        ini_set('max_execution_time', 720000);
        ini_set("default_charset", "utf-8");
        ini_set("memory_limit", "-1");
        ini_set("display_errors", "on");
        ob_start();

        $this->setOrderIds();
        $this->setPostData();

        $this->processingExport();
    }

    public function processingExport(){


        Mage::log('EXPORT STARTED',NULL,'shipmentexport.log');
        Mage::app()->setCurrentStore(Mage_Core_Model_App::ADMIN_STORE_ID);

        $tokenData = Mage::helper('packs_magento')->Authorize();
        $this->tToken = $tokenData['type'];
        $this->aToken = $tokenData['token'];
        Mage::log('time 0: Authorize : ' . $this->getSeconds().' sec',NULL,'shipmentexport.log');

        if($this->continue == true){
            $this->getOrderDataFromMagento();
            Mage::log('time 1: DATA FROM MAGENTO : ' . $this->getSeconds().' sec',NULL,'shipmentexport.log');
        }

        if($this->continue == true) {
            $this->sendShipmentDataToWebservices();
            Mage::log('time 2: DATA TO WEBSERVICES : ' . $this->getSeconds() . ' sec', NULL, 'shipmentexport.log');
        }

        if($this->continue == true) {
            $this->responseHandler();
            Mage::log('time 3: RESPONSE HANDLER : ' . $this->getSeconds().' sec',NULL,'shipmentexport.log');
        }

        if($this->continue == true) {
            $this->createMagentoShipments();
            Mage::log('time 4: CREATE SHIPMENTS IN MAGENTO : ' . $this->getSeconds() . ' sec', NULL, 'shipmentexport.log');
        }

        $this->returnMessages();
        Mage::log('time 5: MESSAGES RETURNED : ' . $this->getSeconds().' sec',NULL,'shipmentexport.log');
    }


    public function getOrderDataFromMagento(){

         $orderCollection = Mage::getModel('sales/order')
            ->getCollection()
            ->addAttributeToFilter('entity_id', array('in' => $this->orderIds));
//            ->addAttributeToSelect('*');

        foreach($orderCollection as $order){
            if($order->canShip())
            {
                $orderData = $order->getData();
                $orderData['shippingaddress'] = $order->getShippingAddress()->getData();

                if($order->getShippingAddress()->getStreet(2)){
                    $orderData['shippingaddress']['street1'] = $order->getShippingAddress()->getStreet(1);
                    $orderData['shippingaddress']['street2'] = $order->getShippingAddress()->getStreet(2);
                    $orderData['shippingaddress']['street3'] = $order->getShippingAddress()->getStreet(3);
                }else{
                    $streetPieces = explode(" ",$orderData['shippingaddress']['street']);
                    $housenumber = end($streetPieces);
                    array_pop($streetPieces);
                    $street = implode(' ', $streetPieces);
                    $orderData['shippingaddress']['street1'] = $street;
                    $orderData['shippingaddress']['street2'] = $housenumber;
                    $orderData['shippingaddress']['street3'] = $nmbrext;
                }
                $zip = str_replace(' ', '', $orderData['shippingaddress']['postcode']);
                $orderData['shippingaddress']['postcode'] = $zip;
                $this->orders[] = $orderData;
            }else{
                $message = 'Order already shipped';
                $this->responseMessages['errors'][$order->getIncrementId()] = $message;
            }
        }
    }

    public function sendShipmentDataToWebservices(){
        $senderData = $this->getSenderData();
        foreach($this->orders as $order){
            try {
            // xml post structure
                $xml_post_array = array(
                    'handler'=> $senderData['handler'],
                    'network'=> $senderData['network'],
                    'loadDate'=> $this->getLoadDate($order),
                    'deliveryDate'=> $this->getDeliveryDate($order),
                    'loadAddress'=> array(
                        'country'=> $senderData['country'],
                        'name'=> $senderData['name'],
                        'street'=> $senderData['street'],
                        'number'=> $senderData['housenumber'],
                        'numberExt'=> $senderData['numbext'],
                        'location'=> '',
                        'zip'=> strtoupper($senderData['postcode']),
                        'place'=> $senderData['city'],
                        'reference'=> $senderData['reference'],
                        'mail'=> $senderData['email'],
                    ),
                    'deliveryAddress'=> array(
                        'country'=> $order['shippingaddress']['country_id'],
                        'name'=> $order['shippingaddress']['firstname'] .' '. $order['shippingaddress']['lastname'],
                        'street'=> $order['shippingaddress']['street1'],
                        'number'=> $order['shippingaddress']['street2'],
                        'numberExt'=> $order['shippingaddress']['street3'],
                        'location'=> '',// ??
                        'zip'=> strtoupper($order['shippingaddress']['postcode']),
                        'place'=> $order['shippingaddress']['city'],
                        'reference'=> $this->getReferentie($order),
                        'mail'=> $order['shippingaddress']['email'],
                        'phone'=> $order['shippingaddress']['telephone'],
                    ),
                    'surcharges'=> array(),
                    'shipmentItems' => $this->getShipmentItems($order),
                );

                $xml_post_string = json_encode($xml_post_array);

                // PHP cURL  for https connection with auth
                $ch = curl_init();
                curl_setopt_array($ch, array(
                    CURLOPT_URL => "".$this->apiUrl."",
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_ENCODING => "",
                    CURLOPT_MAXREDIRS => 10,
                    CURLOPT_TIMEOUT => 30,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => "POST",
                    CURLOPT_POSTFIELDS => "".$xml_post_string."",
                    CURLOPT_HTTPHEADER => array(
                        "Accept: application/json",
                        "Authorization: ".$this->tToken." ".$this->aToken,
                        "Content-Type: application/json",
                        "cache-control: no-cache",
                    ),
                    CURLOPT_SSL_VERIFYPEER => false,

                ));

                $response = curl_exec($ch);

                if(empty($response)){
                    $response = curl_error($ch);
                    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                    curl_close($ch);
                    unset($ch);
                    $this->continue = false;
                    Mage::getSingleton('core/session')->addError('Api response is empty');

                }else{

                    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                    if($httpCode == 404) {
                        $message = "HTTP Error 404. The requested resource is not found. URL";
                        $this->responseMessages['errors'][$order['increment_id']] = $message;

                    }
                    if($httpCode == 400) {
                        $message = $response;
                        $this->responseMessages['errors'][$order['increment_id']] = $message;
                        $this->continue = false;

                    }
                    if($httpCode == 503) {
                        $message = "HTTP Error 503. The api server is not available";
                        $this->responseMessages['errors'][$order['increment_id']] = $message;
                        $this->continue = false;

                    }
                    if (curl_errno($ch)) {
                        $err = curl_error($ch);
                        curl_close($ch);

                        $message = $err;
                        $this->responseMessages['errors'][$order['increment_id']] = $message;

                    }

                    $jsonDecoded = json_decode($response,true);
                    curl_close($ch);
                    unset($ch);

                    $this->responses[$order['increment_id']] = $jsonDecoded;
                }

            }catch (Exception $e) {
                Mage::log('time 4.1: ' . time(). ' : '.$e->getMessage(),NULL,'shipmentexport.log');
                Mage::logException($e);

                // send email
                if($this->sendErrorMail == true){
                    $this->helper->sendErrorMail($e->getMessage(),$this->sendErrorMailTo);
                }
            }
        }
    }

    public function getShipmentItems($order){
        $shipmentItems = array();
        $colliQty = (int)$this->getCollie($order);

        for($i=0; $i<$colliQty; $i++){
            $shipmentItem = array(
                'product' => $this->getSeal($order),
                'weight' => (int)$this->getTotalWeight($order)/$colliQty,
                'length'=> 1,
                'height'=> 1,
                'width'=> 1,
                'labelText'=> 'collo'.$i,
                'surcharges'=> array($this->getShipmentItemSurcharges($order))
            );
            array_push($shipmentItems,$shipmentItem);
        }

        return $shipmentItems;
    }

    private function getShipmentItemSurcharges($order){
        if($this->getAllowance($order)) {
            $surcharge = array('surcharge'=> (string)$this->getAllowance($order));
        }else{
            $surcharge = '';
        }
        return $surcharge;
    }

    public function responseHandler(){
        foreach($this->responses as $incrementId => $xml){
            if((string)$xml->Body->Fault->faultcode){
                $pieces = explode('--->',(string)$xml->Body->Fault->faultstring);
                $pieces = explode('bij Shipment.InsertShipment',$pieces[1]);
                $message = $pieces[0];
                $this->responseMessages['errors'][$incrementId] = $message;
            }
            if((string)$xml->Body->InsertShipmentBasicResponse->InsertShipmentBasicResult->diffgram->NewDataSet->CheckBalance->error){
                $message = (string)$xml->Body->InsertShipmentBasicResponse->InsertShipmentBasicResult->diffgram->NewDataSet->CheckBalance->error;
                $this->responseMessages['errors'][$incrementId] = $message;
            }

            if(isset($this->responseMessages['errors'])){
                if(array_key_exists($incrementId, $this->responseMessages['errors'])){
                    unset($this->responses[$incrementId]);
                }
            }
        }
    }

    public function createMagentoShipments(){

        foreach($this->responses as $incrementId => $data){
            $order = Mage::getModel('sales/order')->loadByIncrementId($incrementId);
            $packsShipmentId = (string)$data['shipmentId'];
            if($packsShipmentId == 0){
                $message = 'No Shipment ID Returned';
                $this->responseMessages['errors'][$incrementId] = $message;
                return;
            }
            $packsShipmentPostcode = (string)$data['deliveryAddress']['zip'];

            $packsShipmentLabels = array();
            foreach($data['shipmentItems'] as $shipmentItem){
                array_push($packsShipmentLabels,$shipmentItem['labelObject']);
            }


            //$trackAndTraceUrl = 'https://www.packs.nl/tracktrace/?zendingnr='.$packsShipmentId.'&pc6hnr='.$packsShipmentPostcode;
            $shippingNotes = 'Shipment added by Packs_Magento extension';
            $itemQty =  $order->getItemsCollection()->count();

            $shipment = Mage::getModel('sales/service_order', $order)->prepareShipment($itemQty);
            $shipment = new Mage_Sales_Model_Order_Shipment_Api();
            $shipmentId = $shipment->create($order->getIncrementId());
            //$shipment->addTrack($shipmentId, 'custom', 'Packs Magento', $trackAndTraceUrl);
            $shipment->addComment($shipmentId, $shippingNotes, false, false);

            $saveData = array(
                'magento_order_id' => $order->getEntityId(),
                'magento_shipment_id' => $shipmentId,
                'packs_shipment_id' => $packsShipmentId,
                'packs_created_at' => Mage::getModel('core/date')->gmtDate('Y-m-d H:i:s'),
                'packs_load_date' => $this->postData[$incrementId.'-loaddate'],
                'packs_delivery_date' => $this->postData[$incrementId.'-deliverydate'],
                'packs_confirm_status' => 'Done',
                'packs_confirm_date' => Mage::getModel('core/date')->gmtDate('Y-m-d H:i:s'),
                'packs_shipment_type' => $this->postData[$incrementId.'-allowance'],
                'packs_seal_type' => $this->postData[$incrementId.'-seal'],
                'packs_collie' => $this->postData[$incrementId.'-collie'],
                'packs_weight' => $this->postData[$incrementId.'-weight'],
                'packs_reference' => $this->postData[$incrementId.'-reference'],
            );
            $packsShipmentModel = Mage::getModel('packs_magento/shipment');
            $packsShipmentModel->setData($saveData);
            $packsShipmentModel->save();

            foreach($data['shipmentItems'] as $shipmentItem){
                $saveItemData = array(
                    'packs_shipment_item_id' =>   $shipmentItem['shipmentItemId'],
                    'magento_order_id' =>   $order->getEntityId(),
                    'magento_shipment_id' =>   $shipmentId,
                    'packs_shipment_id' =>   $packsShipmentId,
                    'seal_type' =>   $shipmentItem['product'],
                    'weight' =>   $shipmentItem['weight'],

                );

                $packsShipmentItemsModel = Mage::getModel('packs_magento/shipmentitems');
                $packsShipmentItemsModel->setData($saveItemData);
                $packsShipmentItemsModel->save();

                $labelData = array(
                    'magento_order_id' => $order->getEntityId(),
                    'magento_shipment_id' => $shipmentId,
                    'packs_shipment_id' => $packsShipmentId,
                    'packs_shipment_item_id' => $shipmentItem['shipmentItemId'],
                    'label' => $packsShipmentLabel,
                    'label_type' => 'PDF',
                );
                $packsShipmentLabelModel = Mage::getModel('packs_magento/labels');
                $packsShipmentLabelModel->setData($labelData);
                $packsShipmentLabelModel->save();
            }

            $message = 'Shipment created';
            $this->responseMessages['success'][$incrementId] = $message;
        }
    }

    public function returnMessages(){
        if(isset($this->responseMessages['success'])){
            foreach($this->responseMessages['success'] as $incrementId => $message){
                Mage::getSingleton('core/session')->addSuccess('#'.$incrementId.' : '.$message);
            }
        }
        if(isset($this->responseMessages['errors'])) {
            foreach ($this->responseMessages['errors'] as $incrementId => $message) {
                Mage::getSingleton('core/session')->addError('#'.$incrementId.' : '.$message);
            }
        }
    }

    public function getSeconds(){
        if(!isset($this->oldTime)){
            $this->oldTime = time();
        }
        $newTime = time();
        $seconds = $newTime - $this->oldTime;
        $this->oldTime = $newTime;
        return $seconds;
    }

    public function setOrderIds()
    {
        $orderIds = Mage::getSingleton('packs_magento/export_data')->getData('orderIds');
        $this->orderIds = $orderIds;
    }

    public function setPostData()
    {
        $postData = Mage::getSingleton('packs_magento/export_data')->getData('postData');
        $this->postData = $postData;
    }

    public function getSenderData(){
        $senderData = array();
        $senderData['country'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_country');
        $senderData['city'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_city');
        $senderData['postcode'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_postcode');
        $senderData['street'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_street');
        $senderData['housenumber'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_housenumber');
        $senderData['numbext'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_numbext');
        $senderData['name'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_name');
        $senderData['contact'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_contact');
        $senderData['handler'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_handler');
        $senderData['network'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_network');
        $senderData['email'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_email');
        $senderData['reference'] = Mage::getStoreConfig('packs_magento_shipments/sender_info/sender_reference');
        return $senderData;
    }

    public function getSeal($order){
        $seal = $this->postData[$order['increment_id'].'-seal'];
        return $seal;
    }

    public function getLoadDate($order){
        $loadDate = $this->postData[$order['increment_id'].'-loaddate'];
        return $loadDate;
    }

    public function getDeliveryDate($order){
        $deliveryDate = $this->postData[$order['increment_id'].'-deliverydate'];
        return $deliveryDate;

    }

    public function getMemo($order){
        /*
         * Hier moet nog logica hoe de memo opgehaalt moet worden
         */
        $memo = '';
        return $memo;
    }

    public function getReferentie($order){
        $reference = $this->postData[$order['increment_id'].'-reference'];
        return $reference;
    }

    public function getCollie($order){
        $collie = $this->postData[$order['increment_id'].'-collie'];
        return $collie;
    }

    public function getTotalWeight($order){
        $weight = $this->postData[$order['increment_id'].'-weight'];
        return $weight;
    }

    public function getAllowance($order){
        /*
         * Hier moet nog logica voor toeslag geschreven worden
         */
        $allowance = $this->postData[$order['increment_id'].'-allowance'];
        return $allowance;
    }

    public function splitAddress($address) {
        // Get everything up to the first number with a regex
        $hasMatch = preg_match('/^[^0-9]*/', $address, $match);
        // If no matching is possible, return the supplied string as the street
        if (!$hasMatch) {
            return array($address, "", "");
        }
        // Remove the street from the address.
        $address = str_replace($match[0], "", $address);
        $street = trim($match[0]);
        // Nothing left to split, return
        if (strlen($address == 0)) {
            return array($street, "", "");
        }
        // Explode address to an array
        $addrArray = explode(" ", $address);
        // Shift the first element off the array, that is the house number
        $housenumber = array_shift($addrArray);
        // If the array is empty now, there is no extension.
        if (count($addrArray) == 0) {
            return array($street, $housenumber, "");
        }
        // Join together the remaining pieces as the extension.
        $extension = implode(" ", $addrArray);
        return array($street, $housenumber, $extension);
    }

}